@@ -1,4 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric #-}
 -- |
 -- Module    : Statistics.Distribution.Uniform
 -- Copyright : (c) 2011 Aleksey Khudyakov
@@ -19,16 +19,28 @@ module Statistics.Distribution.Uniform
     , uniformB
     ) where
 
-import Data.Typeable (Typeable)
+import Data.Aeson (FromJSON, ToJSON)
+import Data.Binary (Binary)
+import Data.Data (Data, Typeable)
+import GHC.Generics (Generic)
 import qualified Statistics.Distribution as D
 import qualified System.Random.MWC       as MWC
+import Data.Binary (put, get)
+import Control.Applicative ((<$>), (<*>))
 
 
 -- | Uniform distribution from A to B
 data UniformDistribution = UniformDistribution {
       uniformA :: {-# UNPACK #-} !Double -- ^ Low boundary of distribution
     , uniformB :: {-# UNPACK #-} !Double -- ^ Upper boundary of distribution
-    } deriving (Eq, Read, Show, Typeable)
+    } deriving (Eq, Read, Show, Typeable, Data, Generic)
+
+instance FromJSON UniformDistribution
+instance ToJSON UniformDistribution
+
+instance Binary UniformDistribution where
+    put (UniformDistribution x y) = put x >> put y
+    get = UniformDistribution <$> get <*> get
 
 -- | Create uniform distribution.
 uniformDistr :: Double -> Double -> UniformDistribution
@@ -37,7 +49,7 @@ uniformDistr a b
   | a < b     = UniformDistribution a b
   | otherwise = error "Statistics.Distribution.Uniform.uniform: wrong parameters"
 -- NOTE: failure is in default branch to guard againist NaNs.
-                
+
 instance D.Distribution UniformDistribution where
   cumulative (UniformDistribution a b) x
     | x < a     = 0
@@ -69,5 +81,11 @@ instance D.MaybeMean UniformDistribution where
 instance D.MaybeVariance UniformDistribution where
     maybeStdDev   = Just . D.stdDev
 
+instance D.Entropy UniformDistribution where
+  entropy (UniformDistribution a b) = log (b - a)
+
+instance D.MaybeEntropy UniformDistribution where
+  maybeEntropy = Just . D.entropy
+
 instance D.ContGen UniformDistribution where
     genContVar (UniformDistribution a b) gen = MWC.uniformR (a,b) gen
